#include <string.h>
#include <time.h>
#include "hdfi.h"
#include "crisSisGetUtilities.h"
#include "utilities.h"

void *getStructs(uint32 secondsSince1996Start, uint32 secondsSince1996Stop,
							char *structName) {
    char path[256];
    struct tm *pTimeStruct;
    char instrumentID, zero = 0;
    char dataFileSuffix[10];
    int day1996Start, day1996Stop, day1996;
    int maxStructCount, structSize, hasHeader, structCount;
    char *pTempStart, *pTemp, *pStructBufferStart, *pStructBuffer;
    uint32 getStampsAfter, getStampsUpTo;
    
    getStampsAfter = getWorkingStartTime(secondsSince1996Start, structName) - 1;
    getStampsUpTo = secondsSince1996Stop;
    /* printf("getStampsAfter = %ld\n", getStampsAfter); */
    /* printf("getStampsUpTo = %ld\n", getStampsUpTo); */

    instrumentID = getInstrumentIDFromStructName(structName);
    getDataFileSuffixFromStructName(dataFileSuffix, structName);

    day1996Start = getDay1996FromSecondsSince1996(getStampsAfter + 1);
    day1996Stop = getDay1996FromSecondsSince1996(getStampsUpTo);
    /* printf("day1996Start = %d\n", day1996Start); */
    /* printf("day1996Stop = %d\n", day1996Stop); */

    if (day1996Start == day1996Stop) {
	    /* easy case.  start and stop in same day. */
	void *pStructs;

	pTimeStruct = getDateFromDay1996(day1996Start);
	generateDataFilePath(path, pTimeStruct, instrumentID, dataFileSuffix);
	pStructs = getSubsetOfDaysStructs(path, structName, getStampsAfter,
								getStampsUpTo);
	return pStructs;
    }

	/* more general case, get structs across multiple days... */

	/* allocate enough memory to put all incoming structs */
    structSize = getStructSizeFromName(structName);
    hasHeader = 1;
    maxStructCount = 0;
    for (day1996 = day1996Start; day1996 <= day1996Stop; day1996++) {
	pTimeStruct = getDateFromDay1996(day1996);
	generateDataFilePath(path, pTimeStruct, instrumentID, dataFileSuffix);
	maxStructCount += getFilesStructCount(path, structSize, hasHeader);
    }
    maxStructCount += 1;	/* terminal struct */
    pStructBufferStart = (char *) calloc(maxStructCount, structSize);
    pStructBuffer = pStructBufferStart;

    structCount = 0;

	/* deal with partial start day */
    pTimeStruct = getDateFromDay1996(day1996Start);
    generateDataFilePath(path, pTimeStruct, instrumentID, dataFileSuffix);
    /* printf("path = %s\n", path); */
    pTempStart = (char *) getSubsetOfDaysStructs(path, structName,
						getStampsAfter, getStampsUpTo);
    pTemp = pTempStart;
    while (getTimeStamp(pTemp, structName)) {
	/* printf("stamp = %ld\n", getTimeStamp(pTemp, structName)); */
	memcpy(pStructBuffer, pTemp, structSize);
	pTemp += structSize;
	pStructBuffer += structSize;
	structCount++;
    }
    free(pTempStart);

	/* deal with any full days between day1996Start and day1996Stop */

    for (day1996 = day1996Start + 1; day1996 < day1996Stop; day1996++) {
	pTimeStruct = getDateFromDay1996(day1996);
	generateDataFilePath(path, pTimeStruct, instrumentID, dataFileSuffix);
	pTempStart = (char *) getSubsetOfDaysStructs(path, structName,
						getStampsAfter, getStampsUpTo);
	pTemp = pTempStart;
	while (getTimeStamp(pTemp, structName)) {
	    memcpy(pStructBuffer, pTemp, structSize);
	    pTemp += structSize;
	    pStructBuffer += structSize;
	    structCount++;
	}
	free(pTempStart);
    }

	/* deal with partial stop day */
    pTimeStruct = getDateFromDay1996(day1996Stop);
    generateDataFilePath(path, pTimeStruct, instrumentID, dataFileSuffix);
    pTempStart = (char *) getSubsetOfDaysStructs(path, structName,
						getStampsAfter, getStampsUpTo);
    pTemp = pTempStart;
    while (getTimeStamp(pTemp, structName)) {
	memcpy(pStructBuffer, pTemp, structSize);
	pTemp += structSize;
	pStructBuffer += structSize;
	structCount++;
    }
    free(pTempStart);

	/* terminal zeroed struct */
    memset(pStructBuffer, zero, structSize);
    pStructBuffer += structSize;	/* reminder only */
    structCount++;

	/* resize allocated memory down to fit */
    /* printf("structSize = %d\n", structSize); */
    /* printf("structCount = %d\n", structCount); */
    pStructBufferStart =
	(char *) realloc(pStructBufferStart, (structSize * structCount));


    return pStructBufferStart;
}
