#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include "hdfi.h"
#include "utilities.h"
#include "crisSisGetUtilities.h"

void *getSubsetOfDaysStructs(char *path, char *structName,
				uint32 getStampsAfter, uint32 getStampsUpTo) {
    char *pStructArrayStart, *pNextStructInArray, *pInBuffer;
    int structSize, hasHeader, structsInFileCount, structsInBuffer;
    int buffersToRead, leftoverStructsToRead, bytesToRead, fd, j;
    int leftOverBytesToRead, goodStructCount, doneReading;
    unsigned long i;
    static char buffer[131072];		/* 128k read buffer */
    uint32 timeStamp;

    /* printf("getStampsAfter = %ld\n", getStampsAfter); */
    /* printf("getStampsUpTo = %ld\n", getStampsUpTo); */

    structSize = getStructSizeFromName(structName);
    /* printf("structSize = %d\n", structSize); */
    hasHeader = 1;
    structsInFileCount = getFilesStructCount(path, structSize, hasHeader);

	/* allocate memory for structs (+1 for terminal struct) */
    pStructArrayStart = (char *) calloc((structsInFileCount + 1), structSize);

	/* if no structs were found, return pointer to single terminal struct */
    if (structsInFileCount == 0) {
	pNextStructInArray = pStructArrayStart;
	memset(pNextStructInArray, (char) 0, structSize);  /* zero out struct */
	return pStructArrayStart;
    }

    if ((fd = open(path, O_RDONLY, 0)) == -1) {
	    /* can't open the file for some reason */
	char CANT_OPEN_ERROR_MSG[] = "Unable to open file %s\n\nExiting.\n\n";
	perror("getSubsetOfDaysStructs\n\t");
	fprintf(stderr, CANT_OPEN_ERROR_MSG, path);
	exit(1);
    }
	/* skip past header */
    lseek(fd, HEADER_BYTES, 0);

    structsInBuffer = sizeof(buffer) / structSize;
    /* printf("structsInBuffer = %d\n", structsInBuffer); */
    bytesToRead = structsInBuffer * structSize;
    /* printf("bytesToRead = %d\n", bytesToRead); */
    buffersToRead = structsInFileCount / structsInBuffer;
    /* printf("buffersToRead = %d\n", buffersToRead); */
    pNextStructInArray = pStructArrayStart;
    goodStructCount = 0;
    doneReading = 0;
    for (i = 0; i < buffersToRead; i++) {
	/* printf("buffer # %ld\n", i); */
	readFromFile(fd, buffer, bytesToRead);
	pInBuffer = buffer;
	for (j = 0; j < structsInBuffer; j++) {
	    /* printf("buffer,struct = %ld,%d\n", i, j); */
	    timeStamp = getTimeStamp(pInBuffer, structName);
	    /* printf("timeStamp = %ld\n", timeStamp); */
	    if (timeStamp <= getStampsAfter) ;	/* do nothing */
	    else if (timeStamp <= getStampsUpTo) {
		    /* copy struct from buffer into good structs array */
		memcpy(pNextStructInArray, pInBuffer, structSize);
		pNextStructInArray += structSize;
		goodStructCount++;
		/* printf("goodStructCount = %d\n", goodStructCount); */
	    }
	    else {
		doneReading = 1;
		break;
	    }
	    pInBuffer += structSize;
	}
	if (doneReading) break;
    }
    if (!doneReading) {
	/* printf("not done reading...\n"); */
	    /* now get the remaining structs (less than full buffer's worth) */
	leftoverStructsToRead =
			structsInFileCount - (buffersToRead * structsInBuffer);
	leftOverBytesToRead = leftoverStructsToRead * structSize;
	readFromFile(fd, buffer, leftOverBytesToRead);
	pInBuffer = buffer;
	for (j = 0; j < leftoverStructsToRead; j++) {
	    /* printf("buffer,struct = %ld,%d\n", i, j); */
	    timeStamp = getTimeStamp(pInBuffer, structName);
	    /* printf("timeStamp = %ld\n", timeStamp); */
	    if (timeStamp <= getStampsAfter) ; /* do nothing */
	    else if (timeStamp <= getStampsUpTo) {
		    /* copy struct from buffer into good structs array */
		memcpy(pNextStructInArray, pInBuffer, structSize);
		pNextStructInArray += structSize;
		goodStructCount++;
		/* printf("goodStructCount = %d\n", goodStructCount); */
	    }
	    else break;

	    pInBuffer += structSize;
	}
    }
    close(fd);

	/* zero the terminal struct */
    memset(pNextStructInArray, (char) 0, structSize);
    pNextStructInArray += structSize;			/* reminder only here */
    goodStructCount++;

	/* cut struct array down to size */
    pStructArrayStart = realloc(pStructArrayStart, goodStructCount*structSize);
    return pStructArrayStart;
}
